@@ -39,10 +39,10 @@ MSG_SESSION_EXPIRED = _("Session ID expired")
 MSG_UNEXPECT_RESP = _("Unexpected response from QNAP API")
 
 
-@utils.retry(exception=exception.ShareBackendException,
-             retries=5)
 def _connection_checker(func):
     """Decorator to check session has expired or not."""
+    @utils.retry(exception=exception.ShareBackendException,
+                 retries=5)
     @functools.wraps(func)
     def inner_connection_checker(self, *args, **kwargs):
         LOG.debug('in _connection_checker')
@@ -177,11 +177,15 @@ class QnapAPIExecutor(object):
         for key in params:
             value = params[key]
             if value is not None:
-                sanitized_params[key] = six.text_type(value)
+                if isinstance(value, list):
+                    sanitized_params[key] = [six.text_type(v) for v in value]
+                else:
+                    sanitized_params[key] = six.text_type(value)
         return sanitized_params
 
     @_connection_checker
-    def create_share(self, share, pool_name, create_share_name, share_proto):
+    def create_share(self, share, pool_name, create_share_name,
+                     share_proto, **kwargs):
         """Create share."""
         LOG.debug('create_share_name: %s', create_share_name)
 
@@ -191,10 +195,12 @@ class QnapAPIExecutor(object):
             'vol_name': create_share_name,
             'vol_size': six.text_type(share['size']) + 'GB',
             'threshold': '80',
-            'dedup': 'off',
-            'compression': '1',
-            'thin_pro': '0',
-            'cache': '0',
+            'dedup': ('sha512'
+                      if kwargs['qnap_deduplication'] is True
+                      else 'off'),
+            'compression': '1' if kwargs['qnap_compression'] is True else '0',
+            'thin_pro': '1' if kwargs['qnap_thin_provision'] is True else '0',
+            'cache': '1' if kwargs['qnap_ssd_cache'] is True else '0',
             'cifs_enable': '0' if share_proto == 'NFS' else '1',
             'nfs_enable': '0' if share_proto == 'CIFS' else '1',
             'afp_enable': '0',
@@ -225,7 +231,8 @@ class QnapAPIExecutor(object):
         if root.find('authPassed').text == '0':
             raise exception.ShareBackendException(msg=MSG_SESSION_EXPIRED)
         if root.find('ES_RET_CODE').text < '0':
-            msg = _('Create share %s failed') % share['display_name']
+            msg = _("Fail to create share %s on NAS.") % create_share_name
+            LOG.error(msg)
             raise exception.ShareBackendException(msg=msg)
 
         vol_list = root.find('func').find('ownContent').find('volumeList')
@@ -319,26 +326,22 @@ class QnapAPIExecutor(object):
         if root.find('authPassed').text == '0':
             raise exception.ShareBackendException(msg=MSG_SESSION_EXPIRED)
 
-        if ('vol_no' in kwargs) or ('vol_label' in kwargs):
-            vol_list = root.find('Volume_Info')
-            vol_info_tree = vol_list.findall('row')
-            for vol in vol_info_tree:
-                LOG.debug('Iterating vol name: %(name)s, index: %(id)s',
-                          {'name': vol.find('vol_label').text,
-                           'id': vol.find('vol_no').text})
-                if 'vol_no' in kwargs:
-                    if kwargs['vol_no'] == vol.find('vol_no').text:
-                        LOG.debug('vol_no:%s',
-                                  vol.find('vol_no').text)
-                        return vol
-                elif 'vol_label' in kwargs:
-                    if kwargs['vol_label'] == vol.find('vol_label').text:
-                        LOG.debug('vol_label:%s', vol.find('vol_label').text)
-                        return vol
-                if vol is vol_info_tree[-1]:
-                    return None
-        else:
-            return res_details['data']
+        vol_list = root.find('Volume_Info')
+        vol_info_tree = vol_list.findall('row')
+        for vol in vol_info_tree:
+            LOG.debug('Iterating vol name: %(name)s, index: %(id)s',
+                      {'name': vol.find('vol_label').text,
+                       'id': vol.find('vol_no').text})
+            if 'vol_no' in kwargs:
+                if kwargs['vol_no'] == vol.find('vol_no').text:
+                    LOG.debug('vol_no:%s',
+                              vol.find('vol_no').text)
+                    return vol
+            elif 'vol_label' in kwargs:
+                if kwargs['vol_label'] == vol.find('vol_label').text:
+                    LOG.debug('vol_label:%s', vol.find('vol_label').text)
+                    return vol
+        return None
 
     @_connection_checker
     def get_specific_volinfo(self, vol_id, **kwargs):
@@ -453,9 +456,11 @@ class QnapAPIExecutor(object):
             raise exception.ShareBackendException(msg=MSG_SESSION_EXPIRED)
         # snapshot not exist
         if root.find('result').text == '-206021':
+            LOG.warning('Snapshot id %s does not exist', snapshot_id)
             return
-        # lun not exist
+        # share not exist
         if root.find('result').text == '-200005':
+            LOG.warning('Share of snapshot id %s does not exist', snapshot_id)
             return
         if root.find('result').text < '0':
             msg = _('Failed to delete snapshot.')
@@ -495,12 +500,14 @@ class QnapAPIExecutor(object):
             'sharename': share_dict['sharename'],
             'old_sharename': share_dict['old_sharename'],
             'vol_size': six.text_type(share_dict['new_size']) + 'GB',
-            'dedup': '0',
-            'compression': '1',
-            'thin_pro': '0',
-            'cache': '0',
+            'dedup': 'sha512' if share_dict['deduplication'] else 'off',
+            'compression': '1' if share_dict['compression'] else '0',
+            'thin_pro': '1' if share_dict['thin_provision'] else '0',
+            'cache': '1' if share_dict['ssd_cache'] else '0',
+            'cifs_enable': '1' if share_dict['share_proto'] == 'CIFS' else '0',
+            'nfs_enable': '1' if share_dict['share_proto'] == 'NFS' else '0',
             'afp_enable': '0',
-            'ftp_enable': '1',
+            'ftp_enable': '0',
             'hidden': '0',
             'oplocks': '1',
             'sync': 'always',
@@ -579,6 +586,52 @@ class QnapAPIExecutor(object):
         if root.find('result').text < '0':
             raise exception.ShareBackendException(msg=MSG_UNEXPECT_RESP)
 
+    @_connection_checker
+    def edit_host(self, hostname, ipv4_list):
+        """Execute edit_host API."""
+        params = {
+            'module': 'hosts',
+            'func': 'apply_sethost',
+            'name': hostname,
+            'ipaddr_v4': ipv4_list,
+            'sid': self.sid,
+        }
+        sanitized_params = self._sanitize_params(params)
+
+        # urlencode with True parameter to parse ipv4_list
+        sanitized_params = urllib.parse.urlencode(sanitized_params, True)
+        url = ('/cgi-bin/accessrights/accessrightsRequest.cgi?%s' %
+               sanitized_params)
+
+        res_details = self._execute_and_get_response_details(self.ip, url)
+        root = ET.fromstring(res_details['data'])
+        if root.find('authPassed').text == '0':
+            raise exception.ShareBackendException(msg=MSG_SESSION_EXPIRED)
+        if root.find('result').text < '0':
+            raise exception.ShareBackendException(msg=MSG_UNEXPECT_RESP)
+
+    @_connection_checker
+    def delete_host(self, hostname):
+        """Execute delete_host API."""
+        params = {
+            'module': 'hosts',
+            'func': 'apply_delhost',
+            'host_name': hostname,
+            'sid': self.sid,
+        }
+        sanitized_params = self._sanitize_params(params)
+
+        sanitized_params = urllib.parse.urlencode(sanitized_params)
+        url = ('/cgi-bin/accessrights/accessrightsRequest.cgi?%s' %
+               sanitized_params)
+
+        res_details = self._execute_and_get_response_details(self.ip, url)
+        root = ET.fromstring(res_details['data'])
+        if root.find('authPassed').text == '0':
+            raise exception.ShareBackendException(msg=MSG_SESSION_EXPIRED)
+        if root.find('result').text < '0':
+            raise exception.ShareBackendException(msg=MSG_UNEXPECT_RESP)
+
     @_connection_checker
     def set_nfs_access(self, sharename, access, host_name):
         """Execute set_nfs_access API."""
