@@ -19,6 +19,7 @@ from oslo_serialization import jsonutils
 import six
 import webob
 
+from manila.api.openstack import api_version_request as api_version
 from manila.api.v2 import share_snapshots
 from manila.common import constants
 from manila import context
@@ -31,6 +32,7 @@ from manila.tests.api.contrib import stubs
 from manila.tests.api import fakes
 from manila.tests import db_utils
 from manila.tests import fake_share
+from manila import utils
 
 MIN_MANAGE_SNAPSHOT_API_VERSION = '2.12'
 
@@ -41,6 +43,8 @@ def get_fake_manage_body(share_id=None, provider_location=None,
         'share_id': share_id,
         'provider_location': provider_location,
         'driver_options': driver_options,
+        'user_id': 'fake_user_id',
+        'project_id': 'fake_project_id',
     }
     fake_snapshot.update(kwargs)
     return {'snapshot': fake_snapshot}
@@ -69,9 +73,11 @@ class ShareSnapshotAPITest(test.TestCase):
             'display_description': 'updated_snapshot_description',
         }
 
-    def test_snapshot_create(self):
+    @ddt.data('1.0', '2.16', '2.17')
+    def test_snapshot_create(self, version):
         self.mock_object(share_api.API, 'create_snapshot',
                          stubs.stub_snapshot_create)
+
         body = {
             'snapshot': {
                 'share_id': 'fakeshareid',
@@ -80,11 +86,11 @@ class ShareSnapshotAPITest(test.TestCase):
                 'description': 'displaysnapdesc',
             }
         }
-        req = fakes.HTTPRequest.blank('/snapshots')
+        req = fakes.HTTPRequest.blank('/snapshots', version=version)
 
         res_dict = self.controller.create(req, body)
 
-        expected = fake_share.expected_snapshot(id=200)
+        expected = fake_share.expected_snapshot(version=version, id=200)
 
         self.assertEqual(expected, res_dict)
 
@@ -135,10 +141,13 @@ class ShareSnapshotAPITest(test.TestCase):
                           req,
                           200)
 
-    def test_snapshot_show(self):
-        req = fakes.HTTPRequest.blank('/snapshots/200')
+    @ddt.data('2.0', '2.16', '2.17')
+    def test_snapshot_show(self, version):
+        req = fakes.HTTPRequest.blank('/snapshots/200', version=version)
+        expected = fake_share.expected_snapshot(version=version, id=200)
+
         res_dict = self.controller.show(req, 200)
-        expected = fake_share.expected_snapshot(id=200)
+
         self.assertEqual(expected, res_dict)
 
     def test_snapshot_show_nofound(self):
@@ -175,13 +184,19 @@ class ShareSnapshotAPITest(test.TestCase):
         }
         self.assertEqual(expected, res_dict)
 
-    def _snapshot_list_summary_with_search_opts(self, use_admin_context):
+    def _snapshot_list_summary_with_search_opts(self, version,
+                                                use_admin_context):
         search_opts = fake_share.search_opts()
+        if (api_version.APIVersionRequest(version) >=
+                api_version.APIVersionRequest('2.36')):
+            search_opts.pop('name')
+            search_opts['display_name~'] = 'fake_name'
         # fake_key should be filtered for non-admin
         url = '/snapshots?fake_key=fake_value'
         for k, v in search_opts.items():
             url = url + '&' + k + '=' + v
-        req = fakes.HTTPRequest.blank(url, use_admin_context=use_admin_context)
+        req = fakes.HTTPRequest.blank(
+            url, use_admin_context=use_admin_context, version=version)
 
         snapshots = [
             {'id': 'id1', 'display_name': 'n1', 'status': 'fake_status', },
@@ -194,10 +209,14 @@ class ShareSnapshotAPITest(test.TestCase):
         result = self.controller.index(req)
 
         search_opts_expected = {
-            'display_name': search_opts['name'],
             'status': search_opts['status'],
             'share_id': search_opts['share_id'],
         }
+        if (api_version.APIVersionRequest(version) >=
+                api_version.APIVersionRequest('2.36')):
+            search_opts_expected['display_name~'] = 'fake_name'
+        else:
+            search_opts_expected['display_name'] = search_opts['name']
         if use_admin_context:
             search_opts_expected.update({'fake_key': 'fake_value'})
         share_api.API.get_all_snapshots.assert_called_once_with(
@@ -211,11 +230,15 @@ class ShareSnapshotAPITest(test.TestCase):
         self.assertEqual(
             snapshots[1]['display_name'], result['snapshots'][0]['name'])
 
-    def test_snapshot_list_summary_with_search_opts_by_non_admin(self):
-        self._snapshot_list_summary_with_search_opts(use_admin_context=False)
-
-    def test_snapshot_list_summary_with_search_opts_by_admin(self):
-        self._snapshot_list_summary_with_search_opts(use_admin_context=True)
+    @ddt.data({'version': '2.35', 'use_admin_context': True},
+              {'version': '2.36', 'use_admin_context': True},
+              {'version': '2.35', 'use_admin_context': False},
+              {'version': '2.36', 'use_admin_context': False})
+    @ddt.unpack
+    def test_snapshot_list_summary_with_search_opts(self, version,
+                                                    use_admin_context):
+        self._snapshot_list_summary_with_search_opts(
+            version=version, use_admin_context=use_admin_context)
 
     def _snapshot_list_detail_with_search_opts(self, use_admin_context):
         search_opts = fake_share.search_opts()
@@ -280,33 +303,37 @@ class ShareSnapshotAPITest(test.TestCase):
     def test_snapshot_list_detail_with_search_opts_by_admin(self):
         self._snapshot_list_detail_with_search_opts(use_admin_context=True)
 
-    def test_snapshot_list_detail(self):
+    @ddt.data('2.0', '2.16', '2.17')
+    def test_snapshot_list_detail(self, version):
         env = {'QUERY_STRING': 'name=Share+Test+Name'}
-        req = fakes.HTTPRequest.blank('/shares/detail', environ=env)
-        res_dict = self.controller.detail(req)
-        expected_s = fake_share.expected_snapshot(id=2)
+        req = fakes.HTTPRequest.blank('/snapshots/detail', environ=env,
+                                      version=version)
+        expected_s = fake_share.expected_snapshot(version=version, id=2)
         expected = {'snapshots': [expected_s['snapshot']]}
+
+        res_dict = self.controller.detail(req)
+
         self.assertEqual(expected, res_dict)
 
-    def test_snapshot_updates_description(self):
+    @ddt.data('2.0', '2.16', '2.17')
+    def test_snapshot_updates_display_name_and_description(self, version):
         snp = self.snp_example
         body = {"snapshot": snp}
+        req = fakes.HTTPRequest.blank('/snapshot/1', version=version)
 
-        req = fakes.HTTPRequest.blank('/snapshot/1')
         res_dict = self.controller.update(req, 1, body)
+
         self.assertEqual(snp["display_name"], res_dict['snapshot']["name"])
 
-    def test_snapshot_updates_display_descr(self):
-        snp = self.snp_example
-        body = {"snapshot": snp}
+        if (api_version.APIVersionRequest(version) <=
+                api_version.APIVersionRequest('2.16')):
+            self.assertNotIn('user_id', res_dict['snapshot'])
+            self.assertNotIn('project_id', res_dict['snapshot'])
+        else:
+            self.assertIn('user_id', res_dict['snapshot'])
+            self.assertIn('project_id', res_dict['snapshot'])
 
-        req = fakes.HTTPRequest.blank('/snapshot/1')
-        res_dict = self.controller.update(req, 1, body)
-
-        self.assertEqual(snp["display_description"],
-                         res_dict['snapshot']["description"])
-
-    def test_share_not_updates_size(self):
+    def test_share_update_invalid_key(self):
         snp = self.snp_example
         body = {"snapshot": snp}
 
@@ -315,6 +342,238 @@ class ShareSnapshotAPITest(test.TestCase):
 
         self.assertNotEqual(snp["size"], res_dict['snapshot']["size"])
 
+    def test_access_list(self):
+        share = db_utils.create_share(mount_snapshot_support=True)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+
+        expected = []
+
+        self.mock_object(share_api.API, 'get',
+                         mock.Mock(return_value=share))
+        self.mock_object(share_api.API, 'get_snapshot',
+                         mock.Mock(return_value=snapshot))
+        self.mock_object(share_api.API, 'snapshot_access_get_all',
+                         mock.Mock(return_value=expected))
+
+        id = 'fake_snap_id'
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % id,
+                                      version='2.32')
+
+        actual = self.controller.access_list(req, id)
+
+        self.assertEqual(expected, actual['snapshot_access_list'])
+
+    @ddt.data(('1.1.1.1', '2.32'),
+              ('1.1.1.1', '2.38'),
+              ('1001::1001', '2.38'))
+    @ddt.unpack
+    def test_allow_access(self, ip_address, version):
+        share = db_utils.create_share(mount_snapshot_support=True)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+
+        access = {
+            'id': 'fake_id',
+            'access_type': 'ip',
+            'access_to': ip_address,
+            'state': 'new',
+        }
+
+        get = self.mock_object(share_api.API, 'get',
+                               mock.Mock(return_value=share))
+        get_snapshot = self.mock_object(share_api.API, 'get_snapshot',
+                                        mock.Mock(return_value=snapshot))
+        allow_access = self.mock_object(share_api.API, 'snapshot_allow_access',
+                                        mock.Mock(return_value=access))
+        body = {'allow_access': access}
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % snapshot['id'],
+                                      version=version)
+
+        actual = self.controller.allow_access(req, snapshot['id'], body)
+
+        self.assertEqual(access, actual['snapshot_access'])
+        get.assert_called_once_with(utils.IsAMatcher(context.RequestContext),
+                                    share['id'])
+        get_snapshot.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext), snapshot['id'])
+        allow_access.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext), snapshot,
+            access['access_type'], access['access_to'])
+
+    def test_allow_access_data_not_found_exception(self):
+        share = db_utils.create_share(mount_snapshot_support=True)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % snapshot['id'],
+                                      version='2.32')
+        body = {}
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller.allow_access, req,
+                          snapshot['id'], body)
+
+    def test_allow_access_exists_exception(self):
+        share = db_utils.create_share(mount_snapshot_support=True)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % snapshot['id'],
+                                      version='2.32')
+        access = {
+            'id': 'fake_id',
+            'access_type': 'ip',
+            'access_to': '1.1.1.1',
+            'state': 'new',
+        }
+        msg = "Share snapshot access exists."
+
+        get = self.mock_object(share_api.API, 'get', mock.Mock(
+            return_value=share))
+        get_snapshot = self.mock_object(share_api.API, 'get_snapshot',
+                                        mock.Mock(return_value=snapshot))
+        allow_access = self.mock_object(
+            share_api.API, 'snapshot_allow_access', mock.Mock(
+                side_effect=exception.ShareSnapshotAccessExists(msg)))
+
+        body = {'allow_access': access}
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller.allow_access, req,
+                          snapshot['id'], body)
+
+        get.assert_called_once_with(utils.IsAMatcher(context.RequestContext),
+                                    share['id'])
+        get_snapshot.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext), snapshot['id'])
+        allow_access.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext), snapshot,
+            access['access_type'], access['access_to'])
+
+    def test_allow_access_share_without_mount_snap_support(self):
+        share = db_utils.create_share(mount_snapshot_support=False)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+
+        access = {
+            'id': 'fake_id',
+            'access_type': 'ip',
+            'access_to': '1.1.1.1',
+            'state': 'new',
+        }
+
+        get_snapshot = self.mock_object(share_api.API, 'get_snapshot',
+                                        mock.Mock(return_value=snapshot))
+        get = self.mock_object(share_api.API, 'get',
+                               mock.Mock(return_value=share))
+
+        body = {'allow_access': access}
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % snapshot['id'],
+                                      version='2.32')
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller.allow_access, req,
+                          snapshot['id'], body)
+
+        get.assert_called_once_with(utils.IsAMatcher(context.RequestContext),
+                                    share['id'])
+        get_snapshot.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext), snapshot['id'])
+
+    def test_allow_access_empty_parameters(self):
+        share = db_utils.create_share(mount_snapshot_support=True)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+
+        access = {'id': 'fake_id',
+                  'access_type': '',
+                  'access_to': ''}
+
+        body = {'allow_access': access}
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % snapshot['id'],
+                                      version='2.32')
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller.allow_access, req,
+                          snapshot['id'], body)
+
+    def test_deny_access(self):
+        share = db_utils.create_share(mount_snapshot_support=True)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+        access = db_utils.create_snapshot_access(
+            share_snapshot_id=snapshot['id'])
+
+        get = self.mock_object(share_api.API, 'get',
+                               mock.Mock(return_value=share))
+        get_snapshot = self.mock_object(share_api.API, 'get_snapshot',
+                                        mock.Mock(return_value=snapshot))
+        access_get = self.mock_object(share_api.API, 'snapshot_access_get',
+                                      mock.Mock(return_value=access))
+        deny_access = self.mock_object(share_api.API, 'snapshot_deny_access')
+
+        body = {'deny_access': {'access_id': access.id}}
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % snapshot['id'],
+                                      version='2.32')
+
+        resp = self.controller.deny_access(req, snapshot['id'], body)
+
+        self.assertEqual(202, resp.status_int)
+        get.assert_called_once_with(utils.IsAMatcher(context.RequestContext),
+                                    share['id'])
+        get_snapshot.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext), snapshot['id'])
+        access_get.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext),
+            body['deny_access']['access_id'])
+        deny_access.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext), snapshot, access)
+
+    def test_deny_access_data_not_found_exception(self):
+        share = db_utils.create_share(mount_snapshot_support=True)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % snapshot['id'],
+                                      version='2.32')
+        body = {}
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller.deny_access, req,
+                          snapshot['id'], body)
+
+    def test_deny_access_access_rule_not_found(self):
+        share = db_utils.create_share(mount_snapshot_support=True)
+        snapshot = db_utils.create_snapshot(
+            status=constants.STATUS_AVAILABLE, share_id=share['id'])
+        access = db_utils.create_snapshot_access(
+            share_snapshot_id=snapshot['id'])
+        wrong_access = {
+            'access_type': 'fake_type',
+            'access_to': 'fake_IP',
+            'share_snapshot_id': 'fake_id'
+        }
+
+        get = self.mock_object(share_api.API, 'get',
+                               mock.Mock(return_value=share))
+        get_snapshot = self.mock_object(share_api.API, 'get_snapshot',
+                                        mock.Mock(return_value=snapshot))
+        access_get = self.mock_object(share_api.API, 'snapshot_access_get',
+                                      mock.Mock(return_value=wrong_access))
+
+        body = {'deny_access': {'access_id': access.id}}
+        req = fakes.HTTPRequest.blank('/snapshots/%s/action' % snapshot['id'],
+                                      version='2.32')
+
+        self.assertRaises(webob.exc.HTTPBadRequest,
+                          self.controller.deny_access, req, snapshot['id'],
+                          body)
+        get.assert_called_once_with(utils.IsAMatcher(context.RequestContext),
+                                    share['id'])
+        get_snapshot.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext), snapshot['id'])
+        access_get.assert_called_once_with(
+            utils.IsAMatcher(context.RequestContext),
+            body['deny_access']['access_id'])
+
 
 @ddt.ddt
 class ShareSnapshotAdminActionsAPITest(test.TestCase):
@@ -344,8 +603,8 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
             share = db_utils.create_share()
             snapshot = db_utils.create_snapshot(
                 status=constants.STATUS_AVAILABLE, share_id=share['id'])
-        req = fakes.HTTPRequest.blank('/v2/fake/snapshots/%s/action' %
-                                      snapshot['id'], version=version)
+        path = '/v2/fake/snapshots/%s/action' % snapshot['id']
+        req = fakes.HTTPRequest.blank(path, script_name=path, version=version)
         return snapshot, req
 
     def _reset_status(self, ctxt, model, req, db_access_method,
@@ -451,19 +710,25 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
             self.resource_name, 'manage_snapshot')
 
     @ddt.data(
-        get_fake_manage_body(name='foo', description='bar'),
-        get_fake_manage_body(display_name='foo', description='bar'),
-        get_fake_manage_body(name='foo', display_description='bar'),
-        get_fake_manage_body(display_name='foo', display_description='bar'),
-        get_fake_manage_body(display_name='foo', display_description='bar'),
+        {'version': '2.12',
+         'data': get_fake_manage_body(name='foo', display_description='bar')},
+        {'version': '2.12',
+         'data': get_fake_manage_body(display_name='foo', description='bar')},
+        {'version': '2.17',
+         'data': get_fake_manage_body(display_name='foo', description='bar')},
+        {'version': '2.17',
+         'data': get_fake_manage_body(name='foo', display_description='bar')},
     )
-    def test_snapshot_manage(self, data):
+    @ddt.unpack
+    def test_snapshot_manage(self, version, data):
         self.mock_policy_check = self.mock_object(
             policy, 'check_policy', mock.Mock(return_value=True))
         data['snapshot']['share_id'] = 'fake'
         data['snapshot']['provider_location'] = 'fake_volume_snapshot_id'
         data['snapshot']['driver_options'] = {}
-        return_snapshot = {'id': 'fake_snap'}
+        return_snapshot = fake_share.fake_snapshot(
+            create_instance=True, id='fake_snap',
+            provider_location='fake_volume_snapshot_id')
         self.mock_object(
             share_api.API, 'manage_snapshot', mock.Mock(
                 return_value=return_snapshot))
@@ -474,19 +739,36 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
             'display_description': 'bar',
         }
 
-        actual_result = self.controller.manage(self.manage_request, data)
+        req = fakes.HTTPRequest.blank(
+            '/snapshots/manage', use_admin_context=True, version=version)
 
+        actual_result = self.controller.manage(req, data)
+
+        actual_snapshot = actual_result['snapshot']
         share_api.API.manage_snapshot.assert_called_once_with(
             mock.ANY, share_snapshot, data['snapshot']['driver_options'])
         self.assertEqual(return_snapshot['id'],
                          actual_result['snapshot']['id'])
+        self.assertEqual('fake_volume_snapshot_id',
+                         actual_result['snapshot']['provider_location'])
+
+        if (api_version.APIVersionRequest(version) >=
+                api_version.APIVersionRequest('2.17')):
+            self.assertEqual(return_snapshot['user_id'],
+                             actual_snapshot['user_id'])
+            self.assertEqual(return_snapshot['project_id'],
+                             actual_snapshot['project_id'])
+        else:
+            self.assertNotIn('user_id', actual_snapshot)
+            self.assertNotIn('project_id', actual_snapshot)
         self.mock_policy_check.assert_called_once_with(
-            self.manage_request.environ['manila.context'],
-            self.resource_name, 'manage_snapshot')
+            req.environ['manila.context'], self.resource_name,
+            'manage_snapshot')
 
     @ddt.data(exception.ShareNotFound(share_id='fake'),
               exception.ShareSnapshotNotFound(snapshot_id='fake'),
-              exception.ManageInvalidShareSnapshot(reason='error'))
+              exception.ManageInvalidShareSnapshot(reason='error'),
+              exception.InvalidShare(reason='error'))
     def test_manage_exception(self, exception_type):
         self.mock_policy_check = self.mock_object(
             policy, 'check_policy', mock.Mock(return_value=True))
@@ -497,10 +779,12 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
             share_api.API, 'manage_snapshot', mock.Mock(
                 side_effect=exception_type))
 
-        if isinstance(exception_type, exception.ManageInvalidShareSnapshot):
+        http_ex = webob.exc.HTTPNotFound
+
+        if (isinstance(exception_type, exception.ManageInvalidShareSnapshot)
+                or isinstance(exception_type, exception.InvalidShare)):
             http_ex = webob.exc.HTTPConflict
-        else:
-            http_ex = webob.exc.HTTPNotFound
+
         self.assertRaises(http_ex,
                           self.controller.manage,
                           self.manage_request, body)
@@ -544,6 +828,29 @@ class ShareSnapshotAdminActionsAPITest(test.TestCase):
             self.unmanage_request.environ['manila.context'],
             self.resource_name, 'unmanage_snapshot')
 
+    def test_snapshot_unmanage_replicated_snapshot(self):
+        self.mock_policy_check = self.mock_object(
+            policy, 'check_policy', mock.Mock(return_value=True))
+        share = {'status': constants.STATUS_AVAILABLE, 'id': 'bar_id',
+                 'has_replicas': True}
+        self.mock_object(share_api.API, 'get', mock.Mock(return_value=share))
+        snapshot = {'status': constants.STATUS_AVAILABLE, 'id': 'foo_id',
+                    'share_id': 'bar_id'}
+        self.mock_object(share_api.API, 'get_snapshot',
+                         mock.Mock(return_value=snapshot))
+
+        self.assertRaises(webob.exc.HTTPConflict,
+                          self.controller.unmanage,
+                          self.unmanage_request,
+                          snapshot['id'])
+        self.controller.share_api.get_snapshot.assert_called_once_with(
+            self.unmanage_request.environ['manila.context'], snapshot['id'])
+        self.controller.share_api.get.assert_called_once_with(
+            self.unmanage_request.environ['manila.context'], share['id'])
+        self.mock_policy_check.assert_called_once_with(
+            self.unmanage_request.environ['manila.context'],
+            self.resource_name, 'unmanage_snapshot')
+
     @ddt.data(*constants.TRANSITIONAL_STATUSES)
     def test_snapshot_unmanage_with_transitional_state(self, status):
         self.mock_policy_check = self.mock_object(
